--- select-msw.c~	Thu Sep 21 10:33:29 2000
+++ select-msw.c	Tue Jan 23 19:38:38 2001
@@ -186,7 +186,8 @@
 mswindows_own_selection (Lisp_Object selection_name,
 			 Lisp_Object selection_value,
 			 Lisp_Object how_to_add,
-			 Lisp_Object selection_type)
+			 Lisp_Object selection_type,
+			 int owned_p /* Not used */)
 {
   HGLOBAL 	hValue = NULL;
   UINT		cfType;
--- console.h~	Wed Jan 17 21:07:12 2001
+++ console.h	Tue Jan 23 19:37:34 2001
@@ -97,7 +97,8 @@
   Lisp_Object (*own_selection_method)(Lisp_Object selection_name,
                                       Lisp_Object selection_value,
                                       Lisp_Object how_to_add,
-                                      Lisp_Object selection_type);
+                                      Lisp_Object selection_type,
+			  int owned_p);
   void (*disown_selection_method)(Lisp_Object selection_name,
                                   Lisp_Object timeval);
   Lisp_Object (*get_foreign_selection_method) (Lisp_Object selection_symbol,
--- select-x.c+	Tue Jan 23 19:42:06 2001
+++ select-x.c	Tue Jan 23 19:39:47 2001
@@ -46,7 +46,8 @@
 static void hack_motif_clipboard_selection (Atom selection_atom,
 					    Lisp_Object selection_value,
 					    Time thyme, Display *display,
-					    Window selecting_window);
+					    Window selecting_window,
+					    int owned_p);
 #endif
 
 #define CUT_BUFFER_SUPPORT
@@ -74,6 +75,9 @@
  */
 int x_selection_timeout;
 
+/* Enable motif selection optimizations. */
+int x_selection_strict_motif_ownership;
+
 
 /* Utility functions */
 
@@ -201,7 +205,8 @@
  */
 static Lisp_Object
 x_own_selection (Lisp_Object selection_name, Lisp_Object selection_value,
-		 Lisp_Object how_to_add, Lisp_Object selection_type)
+		 Lisp_Object how_to_add, Lisp_Object selection_type,
+		 int owned_p)
 {
   struct device *d = decode_x_device (Qnil);
   Display *display = DEVICE_X_DISPLAY (d);
@@ -233,7 +238,7 @@
 
 #ifdef MOTIF_CLIPBOARDS
   hack_motif_clipboard_selection (selection_atom, selection_value,
-				  thyme, display, selecting_window);
+				  thyme, display, selecting_window, owned_p);
 #endif
   return selection_time;
 }
@@ -249,7 +254,8 @@
 				Lisp_Object selection_value,
 				Time thyme,
 				Display *display,
-				Window selecting_window)
+				Window selecting_window,
+				int owned_p)
 {
   struct device *d = get_device_from_display (display);
   /* Those Motif wankers can't be bothered to follow the ICCCM, and do
@@ -269,7 +275,10 @@
 	 to look up the new value, and you can't Copy from a buffer, Paste
 	 into a text field, then Copy something else from the buffer and
 	 paste it into the text field -- it pastes the first thing again. */
-/*      && !owned_p */
+      && (!owned_p 
+	  /* Selectively re-enable this because for most users its
+	     just too painful - especially over a remote link. */
+	  || x_selection_strict_motif_ownership)
       )
     {
 #ifdef MOTIF_INCREMENTAL_CLIPBOARDS_WORK
@@ -1771,6 +1780,17 @@
 \"*selectionTimeout\" resource (which is expressed in milliseconds).
 */ );
   x_selection_timeout = 0;
+
+  DEFVAR_BOOL ("x-selection-strict-motif-ownership", &x_selection_strict_motif_ownership /*
+*If true and XEmacs already owns the clipboard, don't own it again in the
+Motif way. Owning the selection on the Motif way does a huge amount of
+X protocol, and it makes killing text incredibly slow when using an
+X terminal.  Howeverm, when enabled Motif text fields don't bother to look up
+the new value, and you can't Copy from a buffer, Paste into a text
+field, then Copy something else from the buffer and paste it into the
+text field; it pastes the first thing again.
+*/ );
+  x_selection_strict_motif_ownership = 1;
 }
 
 void
